// test model for demography (not calibrated, with capital, capital share different in non-tradable and tradable, with capital control tax)
// variable notation: Dx=x_{t}-x_{t-1}, TyTx=x_{t}/y_{t} 
// example: TA means detrended with productivity, TNw means detrended with working population
// Note: the nontradable sector has a trend that differs from others. If capital grows with trend AN_{w}, Y_{N} grows with trend term A^{zetaN}N_{w}, P_{N} grows with trend A^{1-zetaN}. TNwTYN, TNwTCN and TATPN is de-trended according to this assumption.

var
Psi 
xiw
xir  
Piw  
Pir  
PiN 
Omega  
TNwTLT 
TNwTLN 
TATwT 
TATwN 
TATPN 
TANwTH 
TANwTYT 
TNwTYN  
TNwTCN 
TANwTCT
TNwTGN 
TANwTGT
TANwTBD 
lambda 
TANwTB 
DA
AAst
R
RK 
Rd
TANwTKT
TANwTKN
TANwTBF
TANwTtax
KYratio
BFratio
X
Xcomp1
Xcomp2;

varexo
n
pargamma;

parameters
pardelta 
parbeta 
parsigma 
Rast 
gast 
alphar 
alphaw
zetaT
zetaN
c 
rhoL 
omega
tauB
tauK
ghome
Gratio
BDratio
GTNratio;

load parmat;
for k=1:length(M_.param_names)
set_param_value(M_.param_names{k},eval(M_.param_names{k})); 
end;

model;

[name = 'demography dynamic']
(1+n)*Psi=(1-omega)+pargamma*Psi(-1);

[name = 'retiree consumption share dynamic']
1/xir=1+pargamma(+1)*parbeta^(parsigma)*(R(+1)/Pir(+1))^(parsigma-1)/xir(+1);

[name = 'worker consumption share dynamic']
1/xiw=1+parbeta^(parsigma)*(Omega(+1)*R(+1)/Piw(+1))^(parsigma-1)/xiw(+1);

[name = 'Omega dynamic equation']
Omega=omega+(1-omega)*(xir/xiw)^(1/(1-parsigma));

[name = 'retiree CPI']
Pir=PiN^(1-alphar);

[name = 'worker CPI']
Piw=PiN^(1-alphaw);

[name = 'labor market allocation']
(TNwTLT^((rhoL+1)/rhoL)+TNwTLN^((rhoL+1)/rhoL))^(rhoL/(rhoL+1))=1;

[name = 'tradable labor share']
TNwTLT*TATwT=(1-zetaT)*TANwTYT;

[name = 'tradable capital share']
RK(+1)*TANwTKT/(DA(+1)*(1+n(+1)))=zetaT*TANwTYT(+1);

[name = 'non=tradable labor share']
TNwTLN*TATwN=(1-zetaN)*TNwTYN*TATPN;

[name = 'non-tradable capital share']
RK(+1)*TANwTKN/(DA(+1)*(1+n(+1)))=zetaN*TNwTYN(+1)*TATPN(+1);

[name = 'non-tradable wage']
(TATwT/TATwN)^(rhoL)=TNwTLT/TNwTLN;

[name = 'non-tradable inflation']
PiN=TATPN/TATPN(-1)*DA^(1-zetaN);

[name = 'endogenous tradable productivity growth']
DA=ghome+c*TNwTLT(-1)*(1-AAst(-1));

[name = 'domestic production frontier dynamic']
AAst/AAst(-1)=DA/(1+gast);

[name = 'tradeble output']
TANwTYT=(TANwTKT(-1)/(DA*(1+n)))^(zetaT)*TNwTLT^(1-zetaT);

[name = 'non-tradable output']
TNwTYN=(TANwTKN(-1)/(DA*(1+n)))^(zetaN)*TNwTLN^(1-zetaN);

[name = 'market clearing for non-tradable']
TNwTYN=TNwTCN+TNwTGN;

[name = 'non-tradable demand']
TATPN*TNwTCN=(1-alphaw)*xiw*((1-lambda)*R*TANwTB(-1)/(DA*(1+n))+TANwTH)+(1-alphar)*xir*lambda*R*TANwTB(-1)/(DA*(1+n));

[name = 'labor earning dynamic']
TANwTH= TATwT*TNwTLT+TATwN*TNwTLN+omega*TANwTH(+1)*DA(+1)/(Omega(+1)*R(+1));

[name = 'market clearing for tradable']
TANwTCT+TANwTGT+(TANwTKN+TANwTKT)-(1-pardelta)*(TANwTKN(-1)+TANwTKT(-1))/(DA*(1+n))=TANwTYT-TANwTBF+Rast*TANwTBF(-1)/(DA*(1+n));

[name = 'tradable demand']
TANwTCT=alphaw*xiw*((1-lambda)*R*TANwTB(-1)/(DA*(1+n))+TANwTH)+alphar*xir*lambda*R*TANwTB(-1)/(DA*(1+n));

[name = 'law of motion for lambda']
lambda*TANwTB(-1)=(1-omega)*TANwTB(-1)+omega*(1-xir(-1))*lambda(-1)*R(-1)*TANwTB(-2)/(DA(-1)*(1+n(-1)));

[name = 'arbitrage between capital and domestic asset']
Rd=(RK+(1-pardelta))*(1-tauK);

[name = 'total asset']
TANwTKT+TANwTKN+TANwTBF+TANwTBD=TANwTB;

[name = 'government debt return']
Rast*(1-tauB*BFratio(-1))=Rd;

[name = 'average return']
R*TANwTB(-1)/(DA*(1+n))=(RK+(1-pardelta))*(1-tauK)*(TANwTKN(-1)+TANwTKT(-1))/(DA*(1+n))+Rast*(1-tauB*BFratio(-1))*TANwTBF(-1)/(DA*(1+n))+Rd*TANwTBD(-1)/(DA*(1+n))-TANwTtax;

[name = 'government expenditure share']
TATPN*TNwTGN+TANwTGT=Gratio*(TANwTYT+TATPN*TNwTYN);

[name = 'government non-tradable tradable share']
TANwTGT/(TATPN*TNwTGN)=GTNratio;

[name = 'government debt supply']
TANwTBD=BDratio*(TANwTYT+TATPN*TNwTYN);

[name = 'government budget constraint']
TANwTtax+(RK+(1-pardelta))*tauK*(TANwTKN(-1)+TANwTKT(-1))/(DA*(1+n))+TANwTBD+Rast*tauB*BFratio(-1)*TANwTBF(-1)/(DA*(1+n))=Rd*TANwTBD(-1)/(DA*(1+n))+(TATPN*TNwTGN+TANwTGT);

KYratio=(TANwTKT+TANwTKN)/(TANwTYT+TATPN*TNwTYN);
BFratio=TANwTBF/(TANwTYT+TATPN*TNwTYN);
X=Xcomp1*Xcomp2*Psi;
Xcomp1=xir/xiw;
Xcomp2=lambda*R*TANwTB(-1)/(DA*(1+n))/((1-lambda)*R*TANwTB(-1)/(DA*(1+n))+TANwTH)/Psi;
end;

//initial value
initval;
n=0.0065;
pargamma=0.8180;
end;
steady;

//end value
endval;
n=-0.0021;
pargamma=0.9601;
end;

steady;
model_diagnostics;
perfect_foresight_setup(periods=1000);
perfect_foresight_solver;
//perfect_foresight_solver(maxit=100,stack_solve_algo=7, solve_algo=0);

